from PlugIns import PlugIn, PlugInGroup, PlugInContainer

class SheetProviderGroup(PlugInGroup):
    """
PlugInGroup that manages the more complex registration needs 
of SheetProvider PlugIns/Containers
    """

    def _PlugInsChanged(self, container, plugins):

        """Update the registry of sheet names and XML namespaces"""

        sr = container.__sheet_registry__ = {}
        xr = container.__xmlns_registry__ = {}

        for plugin in plugins:

            names, namespaces = plugin.namesForRegistration(container)
            sr[n] = sr.get(n,[]); xr[n]

            for n in names:
                if sr.has_key(n): sr[n].append(plugin)
                else:             sr[n]=[plugin]

            for n in namespaces: 
                if xr.has_key(n): xr[n].append(plugin)
                else:             xr[n]=[plugin]














class SheetProviderContainer:

    SheetProviders = SheetProviderGroup(
        'SheetProviders',
        ['Property Sheet Provider'],
        title = 'Property Plugins',
        attr  = '__sheetProviders__'
    )

    __plugin_groups__ = (SheetProviders,)

    __sheet_registry__ = {}
    __xmlns_registry__ = {}


    def PropertySheetFor(self, client, name='', xmlns=''):
        
        if xmlns:  k,d = xmlns, self.__xmlns_registry__
        elif name: k,d = name,  self.__sheet_registry__
        else:      return None

        self = self.aq_inner    # is this really necessary?

        for sp in d.get(k,[]) + d.get('*',[]):
            sp = sp.__of__(self)
            s = sp.PropertySheetFor(client,name=name,xmlns=xmlns)
            if s is not None: return s

        psf = getattr(self.aq_parent,'PropertySheetFor',None)

        if psf is not None:
            return psf(client,name=name,xmlns=xmlns)


    def PropertySheetsFor(self, client):
        """XXX Return a PropertySheets object - kind of difficult :("""
        pass




class SheetProvider(PlugIn):

    """Sheet provider base class (more of an interface, actually...)"""

    def namesForRegistration(self,container):
        """
Return a tuple (names,namespaces) where names is a sequence of
sheet names, and namespaces a sequence of XML namespaces, which this sheet
provider supports.  '*' may be included in either or both sequences, which
means the provider will be called if a more specific provider could not
be found for a given sheet.  This is mainly so that sheet providers can
potentially create propertysheets dynamically based on the requested name
or namespace.
        """

        return (),()


    def PropertySheetFor(self,client,name='',xmlns=''):
        """
Return client's propertysheet matching name or xmlns, if available.  Otherwise
return None
        """
        pass
